var https = require('https');

exports.handler = (event, context, callback) => {

	try {
		if (event.session.new) {
			console.log("NEW SESSION");
			onSessionStarted(event.request.requestId);
		}

		switch (event.request.type) {
			case "LaunchRequest":
				console.log("LAUNCH REQUEST");
	            onLaunch(event.request.requestId, callback);
				break;
			case "IntentRequest":
				console.log("INTENT REQUEST");
	            onIntent(event.request, callback);
				break;
			case "SessionEndedRequest":
				console.log("SESSION END REQUEST");
				onSessionEnded(event.request.requestId);
				break;
			default:
				context.fail("INVALID REQUEST TYPE:" + event.request.type);
		}
	} catch (error) {
		context.fail("Exception: " + error);
	}
};


function onSessionStarted(requestId) {
	console.log('In function onSessionStarted: ' + requestId);
}

function onLaunch(requestId, callback) {
	console.log('In function onLaunch: ' + requestId);

	handleWelcomeResponse(callback);
}


function onIntent(request, callback) {
	const intentName = request.intent.name;

	console.log('In function onIntent: ' + intentName);

	if (intentName === 'GetStockInfo') {
		getStockInfo(request, callback);
	} else if (intentName === "AMAZON.HelpIntent") {
		handleWelcomeResponse(callback);
	} else if (intentName === "AMAZON.StopIntent" || intentName === "AMAZON.CancelIntent") {
		handleGoodbyeResponse(callback);
	} else {
		throw new Error("Could not indentify indent: " + intent.name);
	}
}


function onSessionEnded(requestId) {
	console.log('In function onSessionEnded: ' + requestId);
}


function getStockInfo(request, callback) {
	console.log('In function getStockInfo');

	fillSlots(request, callback);

	// If fillSlots() has not triggered the prompts required to
	// fill the slots i.e. all slots have been filled then
	// we get here
	var slots = request.intent.slots;

	var stockName = slots.StockName.value;
	var stockDay = slots.StockDay.value;
	var priceType = slots.PriceType.value;

	console.log('All slots filled');

	getStockPrice(stockName, stockDay, priceType, callback);
}


function fillSlots(request, callback) {
	console.log('In function fillSlots');
	console.log('Dialog state: ' + request.dialogState);
	console.log('Intent: ' + request.intent);

	if (request.dialogState !== "COMPLETED") {
	    console.log('Dialog.Delegate directive to prompt for more information');
		const speechletResponse = {
			outputSpeech: null,
			card: null,
			reprompt: null,
			shouldEndSession: false,
			directives:  [{
				type: "Dialog.Delegate"
			}]
		};

		callback(null, generateResponse(speechletResponse, {}));
	}

	// If the dialog is complete then we can move on to handling the intent
	// just return from here
}

function getStockPrice(stockName, stockDay, priceType, callback) {
    console.log('In function getStockPrice');

    console.log("Stock name: " + stockName);
    console.log("Stock day: " + stockDay);
    console.log("Price type: " + priceType);

    var tickerMap = {
    	"apple": "AAPL",
    	"vmvare": "VMW",
    	"ibm": "IBM",
    	"google": "GOOG",
    	"amazon": "AMZN"
    };
    
    var priceMap = {
    	"opening": "open_price",
    	"closing": "close_price",
    	"maximum": "high_price",
    	"high": "high_price",
    	"low": "low_price",
    	"minimum": "low_price"
    };

	var stockTicker = tickerMap[stockName.toLowerCase()];
	var priceTypeCode = priceMap[priceType.toLowerCase()];

	pathString = "/historical_data?ticker=" + stockTicker +
		"&item=" + priceTypeCode +
		"&start_date=" + stockDay +
		"&end_date=" + stockDay;

    console.log('Path string: ' + pathString);
	
	var username = "FAKE_USERNAME";
	var password = "FAKE_PASSWORD";
	var auth = "Basic " + new Buffer(username + ":" + password).toString('base64');

	var request = https.get({
		host: "api.intrinio.com",
		port: 443,
		path: pathString,
		headers: {
			"Authorization": auth
		}
	}, function(response) {
		var json = "";
		response.on('data', function(chunk) {
		    console.log("Received json response: " + chunk);
			json += chunk;
		});

		response.on('end', function() {
			var jsonData = JSON.parse(json);
			var stockPrice = jsonData.data[0].value;

		    console.log("The stock price received is: " + stockPrice);

			const title = "Stock Price";
			var speechOutput = stockName + " " + priceType + 
				" price was at " + stockPrice + " for " + stockDay;
			const speechletResponse = buildSpeechletResponse(title, speechOutput, null, true);	

			callback(null, generateResponse(speechletResponse, {}));
		});
	});
}


// Invoked by the session launch as well as the help intent
function handleWelcomeResponse(callback) {
	console.log('In function handleWelcomeResponse');

	const title = "Welcome";
	const speechOutput = "What stock would you like information on? " + 
		"I have information on Apple, Google, VMVare, IBM and Amazon";

	const repromptText = "I'm sorry I didn't get you. What stock would you like information on? " + 
		"I have information on Apple, Google, VMVare, IBM and Amazon";

	const speechletResponse = buildSpeechletResponse(title, speechOutput, repromptText, false);	
	const sessionAttributes = {};

	callback(null, generateResponse(speechletResponse, sessionAttributes));
}

function handleGoodbyeResponse(callback) {
	console.log('In function handleGoodbyeResponse');

	const title = "Goodbye";
	const speechOutput = "Thank you for trying the Stock Tracker Dialog. Have a nice day! ";

	const speechletResponse = buildSpeechletResponse(title, speechOutput, null, true);	
	const sessionAttributes = {};

	callback(null, generateResponse(speechletResponse, sessionAttributes));
}

buildSpeechletResponse = (title, outputText, repromptText, shouldEndSession) => {
	return {
		outputSpeech: {
			type: "PlainText",
			text: outputText
		},
		card: {
			type: "Simple",
			title: title,
			content: outputText
		},
		reprompt: {
			outputSpeech: {
				type: "PlainText",
				text: repromptText
			},
		},
		shouldEndSession: shouldEndSession
	};
}


generateResponse = (speechletResponse, sessionAttributes) => {

	return {
		version: "1.0",
		sessionAttributes: sessionAttributes,
		response: speechletResponse
	};
}
